use telers::{
    event::{telegram::HandlerResult, EventReturn},
    types::{Message, User},
    Bot,
};

use tokio::time::{sleep, Duration as DurationSleep};

use crate::{
    types::structs::{bot_entity::BotEntity, message_sender::MessageSender},
    utils::telegram::try_do_action::try_restrict,
};

use crate::utils::telegram::ban_member::ban_chat_member;

pub async fn ban(bot_entity: BotEntity, delay: u64) -> HandlerResult {
    let (bot, message, sender): (Bot, Message, MessageSender) = (
        bot_entity.bot_instance,
        bot_entity.receive_message,
        bot_entity.message_sender,
    );

    let reply_to: Message;
    let chat_id: i64 = message.chat().id();

    if let Some(msg) = message.reply_to_message() {
        reply_to = msg.clone();
    } else {
        sender
            .reply_to(message.id())
            .text("Ответьте на сообщение, чтобы забанить.")
            .send(&bot)
            .await?;
        return Ok(EventReturn::Cancel);
    }

    let user_data: &User = reply_to.from().unwrap();
    let user_id: i64 = user_data.id;

    sleep(DurationSleep::from_millis(delay)).await;

    let future = || async { ban_chat_member(&bot, user_id, chat_id).await };
    let demote_args: (&Bot, i64, i64) = (&bot, user_id, chat_id);
    let failure_sender: MessageSender = sender
        .clone()
        .text("Невозможно забанить участника чата, демотните и попробуйте снова");

    if try_restrict(future, demote_args, failure_sender)
        .await
        .is_err()
    {
        Ok(EventReturn::Cancel)
    } else {
        let banned_user_name: String = reply_to.from().unwrap().clone().username.unwrap();

        sender
            .reply_to(message.id())
            .text(format!("Пользователь {} забанен.", banned_user_name))
            .send(&bot)
            .await?;

        Ok(EventReturn::Finish)
    }
}
