use telers::{
    event::{telegram::HandlerResult, EventReturn},
    filters::CommandObject,
    methods::RestrictChatMember,
    types::User,
    types::{ChatPermissions, Message},
    Bot,
};

use crate::types::structs::message_sender::MessageSender;

pub async fn unmute(bot: Bot, message: Message, command: CommandObject) -> HandlerResult {
    let sender = MessageSender::new(message.chat().id());

    let reply_to: Message;
    let chat_id: i64 = message.chat().id();

    if let Some(msg) = message.reply_to_message() {
        reply_to = msg.clone();
    } else {
        sender
            .reply_to(message.id())
            .text("Ответьте на сообщение, чтобы замьютить.")
            .send(&bot)
            .await?;
        return Ok(EventReturn::Cancel);
    }

    let user_data: &User = reply_to.from().unwrap();
    let user_id: i64 = user_data.id;

    let default_member_permissions = ChatPermissions::all()
        .can_change_info(false)
        .can_manage_topics(false)
        .can_invite_users(false)
        .can_pin_messages(false);

    let bot_action = RestrictChatMember::new(chat_id, user_id, default_member_permissions);

    let username = reply_to.from().unwrap().clone().username.unwrap();

    if let Err(error) = bot.send(bot_action).await {
        sender
            .text(format!(
                "Невозможно снять мьют с участника чата по причине: {error:?}"
            ))
            .send(&bot)
            .await?;
        return Ok(EventReturn::Cancel);
    } else {
        sender
            .reply_to(message.id())
            .text(format!("С пользователя {} был снят мьут.", username))
            .send(&bot)
            .await
            .unwrap();
    }

    Ok(EventReturn::Finish)
}
