use core::convert::Into;
use telers::{
    errors::session::ErrorKind,
    methods::SendMessage,
    types::{Message, ReplyParameters},
    Bot,
};

#[derive(Clone)]
pub struct MessageSender {
    chat_id: i64,
    send_message: SendMessage,
}

impl MessageSender {
    pub fn new(group_id: i64) -> Self {
        Self {
            chat_id: group_id,
            send_message: SendMessage::new(group_id, ""),
        }
    }
    pub fn group_id(mut self, val: i64) -> Self {
        self.chat_id = val;
        self.send_message = self.send_message.clone().chat_id(val);
        self
    }
    pub fn text(mut self, val: impl Into<String>) -> Self {
        self.send_message = self.send_message.clone().text(val);
        self
    }
    pub fn disable_notification(mut self, val: bool) -> Self {
        self.send_message = self.send_message.clone().disable_notification(val);
        self
    }
    pub fn reply_to(mut self, msg_id: i64) -> Self {
        let reply = ReplyParameters::new(msg_id).chat_id(self.chat_id);
        self.send_message = self.send_message.clone().reply_parameters(reply);
        self
    }
    pub async fn send(self, bot: &Bot) -> Result<Message, ErrorKind> {
        bot.send(self.send_message).await
    }
}
